;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x7eec7a419750c44af7c4dddaa19d2b55 . "./code.cg2js/cg-common-control.fasl")
(#x221a0fa25cc4f8f891d02331a63a1dbd . "./code.cg2js/cg-header-control.fasl")
(#x46b61b178d200fcdeef72aaa02476bcb . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3334fb6d5f91c08e8bdcdcd9943a8439 . "./code.cg2js/cg-yes-no-string.fasl")
(#x8d474e48893bd509b79c34aa2dd90758 . "./code.cg2js/cg-clipboard.fasl")
(#x8b09076df887f54a0141fe0d359b348c . "./code.cg2js/cg-dde-utils.fasl")
(#x9b98d4ed97530a980fb29d6ecda32a40 . "./code.cg2js/cg-editable-text.fasl")
(#x3a767c4fb7373fc328781e07e2b4feca . "./code.cg2js/cg-text-or-combo.fasl")
(#x120373a06cf48d9279808ac3a0241506 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd1a8a1874cb678c026c56be751a5a4eb . "./code.cg2js/cg-graphics-context.fasl")
(#xce847c79cc7abebea69529f301ad71ac . "./code.cg2js/cg-choose-printer.fasl")
(#x1db423bdbb14dc99384271f9d8ad5d2f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x8ea24d18710b46d9d56549c0a7960c75 . "./code.cg2js/cg-edit-in-place.fasl")
(#xe1903f3f9d9c19c78d14d6adc659bf55 . "./code.cg2js/cg-split-bar.fasl")
(#x42d08e69280fd8cf5dc6c4d05f563860 . "./code.cg2js/cg-timer.fasl")
(#x1f4d962c7a99d88d50fb6436beff044a . "./code.cg2js/cg-progress-indicator.fasl")
(#xc7e38eb132452b9c53e0ecb39fecd166 . "./code.cg2js/cg-object-editor.fasl")
(#xc12cd73eed6f19b29790536120d96935 . "./code.cg2js/cg-fill-texture.fasl")
(#x57e5af46ff3d70b9a44b7d2befb32380 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x9c62f605d92580c9a124378b80b444fe . "./code.cg2js/cg-os-window.fasl")
(#xc534d4e063a0f0b59c0222f9b7860324 . "./code.cg2js/cg-color-dialog.fasl")
(#xa2095700edcaf8135b5bc380ac9fe56b . "./code.cg2js/cg-combo-box.fasl")
(#x7c445a4e2a488ef06e154791fadaf8fc . "./code.cg2js/cg-directory-dialog.fasl")
(#x7f4abbbebc99279c9ef2618b187a4a39 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xf51b5d74b1bbfc7dcfc3cbbe1b738e33 . "./code.cg2js/cg-tooltip.fasl")
(#x21ad9631a93ab3533014b6e65513d1d3 . "./code.cg2js/cg-lettered-menu.fasl")
(#x47d2f7c6169b57997523f0d55a0569e8 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xabd40e4204331ac8d161f9a818a1563e . "./code.cg2js/cg-pixmap.fasl")
(#x3311858abb759f050b1e34e40a35f3aa . "./code.cg2js/cg.fasl")
(#x8a6d254c3fceba2815af73e144abbc6e . "./code.cg2js/cg-class-slot-grid.fasl")
(#x7acdf8b405ba2944c4457cf04c304023 . "./code.cg2js/cg-template-string.fasl")
(#x2b58a2aff7685894410ba31806ca72b9 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xf1f3a3780e9616e3bc8ecb5a47d2903a . "./code.cg2js/cg-check-box.fasl")
(#x8750d8cb7c186bbe18dd27b7571d42bc . "./code.cg2js/cg-toggling-widget.fasl")
(#x2870c7f88b3e3ef83cf4b72a62559a54 . "./code.cg2js/cg-radio-button.fasl")
(#x9ee6dfc71dc1869c94007a2b63285a86 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x757572a06a21f205770b123913534934 . "./code.cg2js/cg-scaling-stream.fasl")
(#x549a85dcc20c5e5a765e793712e91cbb . "./code.cg2js/cg-multi-picture-button.fasl")
(#x4d9f541cd93b7d98d94ccc24af79498f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x5b337a8ec756bea73eabecc763ca38fc . "./code.cg2js/cg-grid-widget.fasl")
(#xa8ab0621ad3cbb1410fdac3306685cf4 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x422f824824367f95a3db15d94498ddf5 . "./code.cg2js/cg-item-list.fasl")
(#xddfeca61d67f4ab285391bbd7396831c . "./code.cg2js/cg-paren-matching.fasl")
(#x046cbf19dba1e60c953e650e8bc2fe6a . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x8616fc8269a929c2dd2286ad509e3c93 . "./code.cg2js/cg-html-dialog.fasl")
(#x3d652f957558feabfd2b2251fd485920 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5cca92ee5c52d9e0f4c75912944b15aa . "./code.cg2js/cg-project-window.fasl")
(#xa4a5a2a0fdefe6510e0a1d697bf5974e . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xc8c013f82c3ac9ed4987153aa032fea1 . "./code.cg2js/cg-toolbar.fasl")
(#x7adb8b113db253d9a2cdfd5efe3a63fa . "./code.cg2js/cg-caret.fasl")
(#x3e1e7e0fafbc4ade56a02c2f5092536f . "./code.cg2js/cg-scroll-bar.fasl")
(#x12550c2b8bf55905561af753dffaf65c . "./code.cg2js/cg-web-browser.fasl")
(#xe34ddfcb4044aa4d009d958bc28e0bd5 . "./code.cg2js/cg-dropping-outline.fasl")
(#xa19428ffadebc811a76ae8a8ce58bfb8 . "./code.cg2js/cg-font-dialog.fasl")
(#x873fca61c8ffbffc23a752343e4ba021 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x3c1c78b9e4a4f7491638cdfc6ee7cd9c . "./code.cg2js/cg-yes-no-list.fasl")
(#xaae92ae6bbf03746473470d641d20d93 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6b24334aedbd4f133b7ff6828d93a44f . "./code.cg2js/cg-lamp.fasl")
(#xc1fe6f9f78765a93a44017411a4d0557 . "./code.cg2js/cg-drawable.fasl")
(#x8f206f05e37b32effad55e2562fee2d4 . "./code.cg2js/cg-choice-list.fasl")
(#x51007a7198c111a09af821616ef8d485 . "./code.cg2js/cg-curve.fasl")
(#x373b2a464e2e4ae7ea09865b17804b9e . "./code.cg2js/cg-base.fasl")
(#xfd5223c6aeead3775a84f506d0e9c877 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x08b397dd5547044f6947c52819bc0bd6 . "./code.cg2js/cg-printing.fasl")
(#xef684e940de663e3c9503ba732e70cb0 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xdc29106b59cb7e3bf082be82e998eacf . "./code.cg2js/cg-hotspot.fasl")
(#xd94589fb6a3ad760c217f1494fb41214 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x264f7437a614de51447517b947df42b9 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe199ada317b2838e589732632f3f94e6 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x54fd11e0f81b72bb98b973439e567007 . "./code.cg2js/cg-comtab.fasl")
(#x52a1712e3d48a2457b6ba59729e905cc . "./code.cg2js/cg-os-widget.fasl")
(#xd1feee40374004a66f7ca4eeddb757a3 . "./code.cg2js/cg-icon.fasl")
(#x97a36a165da698b2d4e8fa6dc216ef79 . "./code.cg2js/cg-dev.fasl")
(#x7325d3eb5df47fcc98f6b5b2cbf69d1d . "./code.cg2js/cg-calendar.fasl")
(#xfa8ffaa881679959274581c4dac87762 . "./code.cg2js/cg-group-box.fasl")
(#xfe86e8e321655f62bf0cbd59a62ea710 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xbff80526d3fa4a56ec377603d96c9de7 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x2535ca4318312463a1f5b2df2c98d0d9 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xac0c691284093e60a6a6d27cc053acd9 . "./code.cg2js/cg-nodes.fasl")
(#x0c9a4fef4fdffbc7928fe853236db100 . "./code.cg2js/cg-intersect.fasl")
(#x15b9eefb91dad3cf2c0c2943859f7c5e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xed424e920498bb93bf4c3b9930f1cb71 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x46e144542a52944274ac84b5effd2754 . "./code.cg2js/cg-class-support.fasl")
(#xec248e054a767fa602b0f7498f876508 . "./code.cg2js/cg-dialog-item.fasl")
(#x743f6496d15705fb86f149c4a429c41e . "./code.cg2js/cg-plot-widget.fasl")
(#x83f1b53e5ee93980d17aa309fa770c7b . "./code.cg2js/cg-pixmap-printing.fasl")
(#xaf673df9ec1327fc16ff5b5840ee0d14 . "./code.cg2js/cg-up-down-control.fasl")
(#x4e0f232c88f02bfa263a546bf4cb12ae . "./code.cg2js/cg-rich-edit.fasl")
(#x70dc72dec41be7db40f48714267da671 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xbb5757c977836f51a697eb631ecffc63 . "./code.cg2js/cg-status-bar.fasl")
(#xe67b45d93b3735b2c566bea5bd888733 . "./code.cg2js/cg-class-grid.fasl")
(#x3b0aad2885f1d0b856237272434479a2 . "./code.cg2js/cg-wrap-string.fasl")
(#xbf01ee47f60be95b397f4710f0a56851 . "./code.cg2js/cg-layout.fasl")
(#x017fadbefb75986c4fd65324e567642c . "./code.cg2js/cg-lisp-widget.fasl")
(#x2658b5820b8af01205e943324e15fc17 . "./code.cg2js/cg-button.fasl")
(#x32f4b0abf5b83449b17b17cd1011ca27 . "./code.cg2js/cg-string-dialog.fasl")
(#x9cdcd9fb213c495653572cad6a31246e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x6a005bcc74b10b2d8746f673cb623554 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xe0c6455e5f04f5a0c71c4362c90861ae . "./code.cg2js/cg-bitmap-stream.fasl")
(#x3bcc171c38555f91ea75a0372bff0f99 . "./code.cg2js/cg-static-text.fasl")
(#x605f50c276cca13c528c25b8eaa8e7b6 . "./code.cg2js/cg-outline.fasl")
(#xe149aaa27dc3a63a7e664c9199b56d14 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x734e99e9da7795ca0ba4498417e33746 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xa85026299ba3381dd41b2aabd1a5e618 . "./code.cg2js/cg-picture-widget.fasl")
(#xff8257426adcf16779f4721fc756f560 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x162c9cab1811b21074d17edda21bc9ad . "./code.cg2js/cg-chart-widget.fasl")
(#xe68bdc71262c3c4501d6687b0f5d2311 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x8856156d37e4d5e51c8d21fe547a342d . "./code.cg2js/cg-lisp-text.fasl")
(#x10c25e8d45c2558d71b833d42f53458f . "./code.cg2js/cg-palette.fasl")
(#x2f4a00405b93c9d713d714229061200e . "./code.cg2js/cg-property.fasl")
(#xaa19081abf88f405bab49d0030d4dfa7 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x419670c3340e00ddea7093a64ff54cf2 . "./code.cg2js/cg-file-dialog.fasl")
(#xa2c29be6a61731e052cd6cb8093b8c4d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xcf36dbf40a162c635fd509d07a79dcb5 . "./code.cg2js/cg-get-pixmap.fasl")
(#x710b186b20ac3f366da5928f0dfa54a6 . "./code.cg2js/cg-tray.fasl")
(#x61b552bd95e1d3ae7f78af7cadd5de5b . "./code.cg2js/cg-acache.fasl")
(#xef8b82a4a1d47246dca941ab97e73041 . "./code.cg2js/cg-message-dialog.fasl")
(#x88ac78fc916954e9b78b178098078b73 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x45e2b9b6e5c381c33831a61dd2b5c60d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x4091e226affff985150579cd727d025b . "./code.cg2js/cg-tab-control.fasl")
(#x839c7c4be5354850c9a659c412d9fe79 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe40e45e136fb8ebdbada70f70691efe7 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x16475bb7128a589ad61150f6a17f23c7 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2a31195a71a99cbf9c3aeb2d171783f4 . "./code.cg2js/cg-menu.fasl")
(#xfa0f037f317bfe1838ec1b3006681b8a . "./code.cg2js/cg-get-position.fasl")
(#x0f5f6715aa236238c70c0f392e0120ce . "./code.cg2js/cg-trackbar.fasl")
(#xf84d9c172df33ad62017c051a960e8af . "./code.cg2js/cg-selected-object.fasl")
(#x0744e31f9ad8b923e1bdc4b426d9bfef . "./code.cg2js/cg-list-view.fasl")
(#xb77f6b978927fef51c50485cbe2cd448 . "./code.cg2js/cg-html-widget.fasl")
(#x4360d52eac87c07247a4b393ce6762a8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xf912ba4571a7c37b3551ca6c014dfb07 . "./code.cg2js/cg-text-widget.fasl")
(#xa6939622b762949537c2f548f5b6e8e3 . "./code.idejs/ide-base.fasl")
(#x7426a5cf1312c80dcc58bd6f3e6bf13f . "./code.idejs/ide-process-browser.fasl")
(#x69adb83be43abf0477c5858db018f4d4 . "./code.idejs/ide-patch-dialog.fasl")
(#xa276f8d4f2217f43a27222ce2bb254e7 . "./code.idejs/ide-keyshort.fasl")
(#x9ae827386441e4fee23bff5f096b3fb2 . "./code.idejs/ide-window-list.fasl")
(#x1eee40ed77fadc2923ea21b633955ea9 . "./code.idejs/ide.fasl")
(#xc52a36adde1bb4eb316245d268f74158 . "./code.idejs/ide-editor.fasl")
(#xf0921e87c55ff8749e9e13b25d5d4f43 . "./code.idejs/ide-clipboard.fasl")
(#x1025518c6d98e8c9af5a67664e598311 . "./code.idejs/ide-trace.fasl")
(#xddb8338686ce7177e18d0a65689a9473 . "./code.idejs/ide-class-browser.fasl")
(#xbeb9d4820329aca5c38ad90f7ba29cb5 . "./code.idejs/ide-grapher.fasl")
(#x5d6fa5e0853dfae3c28cd94e51f25c18 . "./code.idejs/ide-apropos.fasl")
(#xdec66eb56251bc3b8db82fece468f3ad . "./code.idejs/ide-navigator.fasl")
(#xd8616315a314b00f522353b26416e3d1 . "./code.idejs/ide-find-definitions.fasl")
(#x3560b1bb240ddb6cb115331920f4692d . "./code.idejs/ide-project.fasl")
(#xa75a97679f83268daad03531961f9ff7 . "./code.idejs/ide-class-grapher.fasl")
(#xe38f874ed9c62d30f64ff00beb65a68c . "./code.idejs/ide-acache.fasl")
(#x4c857ad48529f621286a84c821c8eeab . "./code.idejs/ide-find-in-files.fasl")
(#xd7fc921dd22a992ae3b7f750037042ed . "./code.idejs/ide-inspector.fasl")
(#xfedd696666051f02af31b50f4d71c353 . "./code.idejs/ide-intro.fasl")
(#xa81e51dd21b4b5a46bcddac64ae64964 . "./code.idejs/ide-package-list.fasl")
(#xd6f3ed718bb3be00d105a161eca29094 . "./code.idejs/ide-dev.fasl")
(#xb9e9c79916cc910f8b3a308d3cc7a789 . "./code.idejs/ide-form.fasl")
(#x785b009207cd19fe54c93fc535e484ec . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
